#include <X11/Xlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <stdio.h>
#include <netdb.h>
#include <pcap.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

long MAXBYTESPERSEC = 6144;

#define HOSTMAX 10
typedef struct {
	char ip[80];
	char name[80];
	long packets;
} hostT;

hostT host[HOSTMAX];
int lasthost = 0;

struct itimerval gfxtimer;

char pc_err[PCAP_ERRBUF_SIZE], *re_err;

struct in_addr net, mask;
pcap_t *pd = NULL;
char *dev = NULL;
int link_offset;
pcap_dumper_t *pd_dump = NULL;

int black, white;
Display *display;
Window win;
GC gc;
	
int height = 0;
int divamount = 0;

int started = 0;

void processpacket(u_char *data1, struct pcap_pkthdr* h, u_char *p) {
	int i;
	struct ip* ip_packet = (struct ip *)(p + 14);
	unsigned ip_hl = ip_packet->ip_hl*4;

	int len;

	len = ntohs(ip_packet->ip_len) - ip_hl;

	for (i = 0; i < lasthost; i++) {
    // check if packet is for this machine
		if (!strcmp(host[i].ip, inet_ntoa(ip_packet->ip_dst))) {
			host[i].packets += len;
		}
  }
}

void initstuff(void) {
	int i;
	for (i = 0; i < HOSTMAX; i++) {
		strcpy(host[i].ip, "0.0.0.0");
		host[i].packets = 0;
	}

	gfxtimer.it_interval.tv_sec = 1;
	gfxtimer.it_interval.tv_usec = 0;

	gfxtimer.it_value.tv_sec = 1;
	gfxtimer.it_value.tv_usec = 0;

	setitimer(ITIMER_REAL, &gfxtimer, 0);

}

void updategfx(int sig) {
	int i;

	/* draw the labels */
	for (i = 0; i < lasthost; i++) {
		char *p;
		int y = 10;
		for (p = host[i].name; *p; p++,y += 13) {
			XDrawString(display, win, gc, i*45+1, y,
									p, 1);
		}
	}

	for (i = 0; i < lasthost; i++) {
		// clear old box
		XSetForeground(display, gc, black);
		XFillRectangle(display, win, gc, i*45+10,0,25,height);
		// draw new
		XSetForeground(display, gc, white);
		XFillRectangle(display, win, gc, i*45+10,height-(host[i].packets / divamount),
																			25, host[i].packets / divamount);

		XFlush(display);
		
		//printf("%s:   %ld\n", host[i].ip, host[i].packets);
		// reset packet counter
    host[i].packets = 0;
  }
	fflush(stdout);
}

void die(const char *txt) {
	printf("%s\n",txt);
	exit(1);
}

void waitforwindow(void) {
	for(;;) {
            XEvent e;
            XNextEvent(display, &e);
            if (e.type == MapNotify)
                  break;
  }

}

int 
main (int argc, char **argv)
{
	int i;
	struct hostent *hoste;
	struct in_addr in;

	signal(SIGALRM, updategfx);

	initstuff();

	if (strstr(argv[0], "ppp")) {
		MAXBYTESPERSEC =  7168;
	} else {	
		MAXBYTESPERSEC =  1280000;
	}

	if (argc > 1) {
		for (i = 0; i < (argc-1); i++) {
			/* is this a hostname? */
			hoste = gethostbyname(argv[i+1]);
			if (hoste == NULL) {
				die("cant get hostname");
			} else {
				bcopy(hoste->h_addr,&in.s_addr,hoste->h_length);

				strcpy(host[i].name, hoste->h_name);	

				strcpy(host[i].ip, inet_ntoa(in));	

				printf("%s (%s)\n",host[i].name,host[i].ip);
			}
		
			if ((strlen(host[i].name)*13 + 30) > height) {
				height = (strlen(host[i].name)*13 + 30);
			}

			lasthost = i+1;
		}
	} else {
		printf("usage: %s ip1 ip2 ip3 ...\n", argv[0]);
		exit(0);
	}

	printf("%ld\n",MAXBYTESPERSEC);
	divamount = MAXBYTESPERSEC / height;

	/* set up xlib */
	if (!(display = XOpenDisplay(0))) {
		die("couldnt open display");
	}
	black = BlackPixel(display, DefaultScreen(display));
	white = WhitePixel(display, DefaultScreen(display));

	win = XCreateSimpleWindow(display, DefaultRootWindow(display),
														0, 0, lasthost*45, height, 0, black, black);

	XSelectInput(display, win, StructureNotifyMask);

	XMapWindow(display, win);

	gc = XCreateGC(display, win, 0, 0);

	XSetForeground(display, gc, white);

	waitforwindow();

	XFlush(display);
	

	/* set up capturing */
	dev = pcap_lookupdev(pc_err);
	if (dev == NULL) {
		perror(pc_err);
		exit(1);
	}

	if ((pd = pcap_open_live(dev, 65535, 1, 1000, pc_err)) == NULL) {
      perror(pc_err);
      exit(-1);
  }

  if (pcap_lookupnet(dev, &net.s_addr, &mask.s_addr, pc_err) == -1) {
      perror(pc_err);
      memset(&net, 0, sizeof(net));
      memset(&mask, 0, sizeof(mask));
  } 

	started = -1;

	pcap_loop(pd, 0, (pcap_handler)processpacket, 0);
	exit(0);

}
